<?php

namespace common\modules\assetManager\controllers;

use Yii;
use yii\data\ActiveDataProvider;
use yii\data\ArrayDataProvider;
use yii\web\Controller;
use yii\web\Response;
use yii\helpers\Url;
use yii\web\NotFoundHttpException;

use common\modules\user\models\User;
use common\modules\assetManager\Module;
use common\modules\assetManager\assets\AssetManagerAsset;
use common\modules\assetManager\models\AssetManager;
use common\modules\assetManager\models\AssetManagerSearch;

/**
 * AssetController implements the CRUD actions for AssetManager model.
 */
class AssetController extends Controller
{
    public $enableCsrfValidation = false;

    /**
     * {@inheritdoc}
     */
    public function behaviors()
    {
        return [
            'verbs' => [
                'class' => \yii\filters\VerbFilter::className(),
                'actions' => [
                    'delete' => ['POST'],
                    'update' => ['post'],
                ],
            ],
            'access' => [
                'class' => \yii\filters\AccessControl::className(),
                'only' => ['index', 'asset-manager', 'upload-manager', 'upload', 'update', 'delete', 'resize', 'info'],
                'rules' => [
                    [
                        'actions' => ['index', 'asset-manager', 'upload-manager', 'upload', 'update', 'delete', 'resize', 'info'],
                        'allow' => true,
                        'roles' => ['@'],
                        'matchCallback' => function ($rule, $action) {
                            return \common\modules\user\components\GenesysUser::isAllowed(Yii::$app->user->identity->role_id, Yii::$app->controller->module->id . '/' . Yii::$app->controller->id);
                        },
                    ],
                ],
            ],
        ];
    }

    public function beforeAction($action)
    {
        if (defined('YII_DEBUG') && YII_DEBUG) {
            Yii::$app->assetManager->forceCopy = true;
        }

        return parent::beforeAction($action);
    }

    /**
     * Lists all AssetManager models.
     * @return mixed
     */
    public function actionIndex()
    {
        return $this->render('index');
    }

    public function actionAssetManager()
    {
        $this->layout = '@common/modules/assetManager/views/layouts/main';

        $model = new AssetManagerSearch();
        $query = new AssetManager();
        $search_query = array();

        if (isset($_GET['AssetManagerSearch']['name'])) {
            $search_query = ['like', 'name', $_GET['AssetManagerSearch']['name']];
        }

        $query = AssetManager::find()->where($search_query)->andWhere(['status' => 'active']);

        $dataProvider = new ActiveDataProvider([
            'query' => $query,
            'pagination' => [
                'pageSize' => 15,
            ],
            'sort' => [
                'defaultOrder' => [
                    'created_at' => SORT_DESC,
                    'name' => SORT_ASC, 
                ]
            ],
        ]);

        return $this->render('asset-manager', [
            'model' => $model,
            'dataProvider' => $dataProvider,
        ]);
    }

    public function actionUploadManager()
    {
        $this->layout = '@common/modules/assetManager/views/layouts/main';
        
        return $this->render('upload-manager', ['model' => new AssetManager()]);
    }

    /**
     * Provides upload file
     * @return mixed
     */
    public function actionUpload()
    {
        Yii::$app->response->format = Response::FORMAT_JSON;

        $model = new AssetManager();

        $routes = $this->module->routes;
        $rename = $this->module->rename;

        $model->saveAsset($routes, $rename, 'assets');

        $bundle = AssetManagerAsset::register($this->view);

        if ($model->isImage()) {
            $model->createThumbs($routes, $this->module->thumbs);
        }

        $response['files'][] = [
            'url'           => $model->path,
            'thumbnailUrl'  => $model->getDefaultThumbUrl($bundle->baseUrl),
            'name'          => $model->name,
            'type'          => $model->mime_type,
            'size'          => $model->file->size,
            'deleteUrl'     => Url::to(['asset/delete', 'id' => $model->id]),
            'deleteType'    => 'POST',
        ];

        return $response;
    }

    /**
     * Updated mediafile by id
     * @param $id
     * @return array
     */
    public function actionUpdate($id)
    {
        $model = AssetManager::findOne($id);
        $message = Module::t('main', 'Changes not saved.');

        if ($model->load(Yii::$app->request->post()) && $model->save()) {
            $message = Module::t('main', 'Changes saved!');
        }

        Yii::$app->session->setFlash('mediafileUpdateResult', $message);

        Yii::$app->assetManager->bundles = false;

        return $this->renderAjax('info', [
            'model' => $model,
            'strictThumb' => null,
        ]);
    }

    /**
     * Delete model with files
     * @param $id
     * @return array
     */
    public function actionDelete($id)
    {
        Yii::$app->response->format = Response::FORMAT_JSON;

        $routes = $this->module->routes;

        $model = AssetManager::findOne($id);

        if ($model->isImage()) {
            $model->deleteThumbs($routes);
        }

        $model->deleteFile($routes);
        $model->delete();

        return ['success' => 'true'];
    }

    /**
     * Resize all thumbnails
     */
    public function actionResize()
    {
        $models = AssetManager::findByTypes(AssetManager::$imageFileTypes);
        
        $routes = $this->module->routes;

        foreach ($models as $model) {
            if ($model->isImage()) {
                $model->deleteThumbs($routes);
                $model->createThumbs($routes, $this->module->thumbs);
            }
        }

        Yii::$app->session->setFlash('successResize');

        $this->redirect(Url::to(['default/settings']));
    }

    /** Render model info
     * @param int $id
     * @param string $strictThumb only this thumb will be selected
     * @return string
     */
    public function actionInfo($id, $strictThumb = null)
    {
        $model = AssetManager::findOne($id);

        Yii::$app->assetManager->bundles = false;
        
        return $this->renderAjax('info', [
            'model' => $model,
            'strictThumb' => $strictThumb,
        ]);
    }

    public function actionAssetList($q = null, $id = null) 
    {
        \Yii::$app->response->format = \yii\web\Response::FORMAT_JSON;
        $out = ['results' => ['id' => '', 'text' => '']];

        if (!is_null($q)) {
            $query = new \yii\db\Query;
            $query->select('id, name AS text')
                ->from('asset')
                ->where(['like', 'name', $q])
                ->limit(20);
            $command = $query->createCommand();
            $data = $command->queryAll();
            $out['results'] = array_values($data);
        } elseif ($id > 0) {
            $out['results'] = ['id' => $id, 'text' => AssetManager::find($id)->name];
        }

        return $out;
    }

    public function actionDownload($filename = '', $dir = '', $auth = false)
    {
        if ($auth) {
            if (!User::find()->where(['auth_key' => $auth, 'status' => 'active'])->exists()) {
                throw new \yii\web\UnauthorizedHttpException("File access not allowed.");
            }
        }

        $file = Yii::getAlias('@assets');

        if (!empty($dir)) {
            $file .= '/' . $dir;
        }

        $file .= '/' . $filename;
        
        if (file_exists($file)) {
            header('Content-Description: File Transfer');
            header('Content-Type: application/octet-stream');
            header('Content-Disposition: attachment; filename="' . basename($file) . '"');
            header('Expires: 0');
            header('Cache-Control: must-revalidate');
            header('Pragma: public');
            header('Content-Length: ' . filesize($file));

            flush(); // Flush system output buffer

            readfile($file);

            exit;
        } else {
            throw new NotFoundHttpException("File not found.");
        }
    }
}
