<?php

namespace common\modules\inventory\controllers\backend;

use Yii;
use yii\data\ActiveDataProvider;
use yii\data\ArrayDataProvider;
use yii\web\Controller;
use yii\web\NotFoundHttpException;

use common\modules\inventory\models\InventoryItem;
use common\modules\inventory\models\InventoryItemSearch;
use common\modules\core\models\CoreLanguage;
use common\modules\core\models\CoreEntityClass;
use common\modules\core\models\CoreAttributeClass;
use common\modules\core\models\CoreEntityAttribute;
use common\modules\core\models\CoreValue;

/**
 * ItemController implements the CRUD actions for InventoryItem model.
 */
class ItemController extends Controller
{
    /**
     * {@inheritdoc}
     */
    public function behaviors()
    {
        return [
            'verbs' => [
                'class' => \yii\filters\VerbFilter::className(),
                'actions' => [
                    'delete' => ['POST'],
                ],
            ],
            'access' => [
                'class' => \yii\filters\AccessControl::className(),
                'only' => ['index', 'view', 'create', 'update', 'delete', 'duplicate'],
                'rules' => [
                    [
                        'allow' => true,
                        'actions' => ['index', 'view', 'create', 'update', 'delete', 'duplicate'],
                        'roles' => ['@'],
                        'matchCallback' => function ($rule, $action) {
                            return \common\modules\user\components\GenesysUser::isAllowed(Yii::$app->user->identity->role_id, Yii::$app->controller->module->id . '/' . Yii::$app->controller->id);
                        },
                    ],
                ],
            ],
        ];
    }

    /**
     * Lists all InventoryItem models.
     * @return mixed
     */
    public function actionIndex()
    {
        $searchModel = new InventoryItemSearch();
        $dataProvider = $searchModel->search(Yii::$app->request->queryParams);

        return $this->render('index', [
            'searchModel' => $searchModel,
            'dataProvider' => $dataProvider,
        ]);
    }

    /**
     * Displays a single InventoryItem model.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionView($id)
    {
        return $this->render('view', [
            'model' => $this->findModel($id),
        ]);
    }

    /**
     * Creates a new InventoryItem model.
     * If creation is successful, the browser will be redirected to the 'view' page.
     * @return mixed
     */
    public function actionCreate()
    {
        $model = new InventoryItem();
        $data = array();
        $item_traits = array();

        $traits = \common\modules\shop\models\ShopTrait::getAll('id', 'name', true, 'trait', 'name');

        $modelEntityClass = CoreEntityClass::findOne(['name' => 'trait_value', 'status' => 'active']);
        $modelAttributeClass = CoreAttributeClass::findOne(['name' => 'name', 'status' => 'active']);

        foreach ($traits as $trait_k => $trait_v) {
            $trait_value_ids = array();

            $item_traits[$trait_k]['name'] = $trait_v;
            $trait_values = \common\modules\shop\models\ShopTraitValue::find()->where(['trait_id' => $trait_k, 'status' => 'active'])->asArray()->all();
        
            foreach($trait_values as $trait_value) {
                array_push($trait_value_ids, $trait_value['id']);
            }
        
            $trait_value_ids = array_values($trait_value_ids);

            $trait_names = CoreValue::find()
                ->select('entity_id, value')
                ->where([
                    'entity_class_id' => $modelEntityClass->id, 
                    'attribute_class_id' => $modelAttributeClass->id, 
                    'entity_id' => $trait_value_ids, 
                    'status' => 'active'
                ])
                ->asArray()
                ->all();
            
            foreach($trait_names as $trait_name) {
                $item_traits[$trait_k]['value'][$trait_name['entity_id']]['name'] = $trait_name['value'];
                $item_traits[$trait_k]['value'][$trait_name['entity_id']]['is_selected'] = 0;
            }
        }

        if (Yii::$app->request->isPost) {
            if ($item_id = $model->saveItem(Yii::$app->request->post())) {
                $eventInventoryEvents = new \common\modules\inventory\components\InventoryEvents;
                $eventInventoryEvents->eventModInventoryLogChange($item_id, 'create');

                return $this->redirect(['view', 'id' => $item_id]);   
            }
        }

        $data['model'] = $model;
        $data['item_traits'] = $item_traits;

        return $this->render('create', [
            'data' => $data,
        ]);
    }

    /**
     * Updates an existing InventoryItem model.
     * If update is successful, the browser will be redirected to the 'view' page.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionUpdate($id)
    {
        $model = $this->findModel($id);
        $data = array();
        $item_traits = array();
        $log_flag = false;

        $traits = \common\modules\shop\models\ShopTrait::getAll('id', 'name', true, 'trait', 'name');

        $modelEntityClass = CoreEntityClass::findOne(['name' => 'trait_value', 'status' => 'active']);
        $modelAttributeClass = CoreAttributeClass::findOne(['name' => 'name', 'status' => 'active']);

        foreach ($traits as $trait_k => $trait_v) {
            $trait_value_ids = array();

            $item_traits[$trait_k]['name'] = $trait_v;

            if(\common\modules\inventory\models\InventoryItemTraitValue::find()->where(['item_id' => $id, 'trait_id' => $trait_k, 'status' => 'active'])->exists()) {
                $item_traits[$trait_k]['is_selected'] = 1;
            } else {
                $item_traits[$trait_k]['is_selected'] = 0;
            }

            $trait_values = \common\modules\shop\models\ShopTraitValue::find()->where(['trait_id' => $trait_k, 'status' => 'active'])->asArray()->all();
        
            foreach($trait_values as $trait_value) {
                array_push($trait_value_ids, $trait_value['id']);
            }
        
            $trait_value_ids = array_values($trait_value_ids);

            $trait_names = CoreValue::find()
                ->select('entity_id, value')
                ->where([
                    'entity_class_id' => $modelEntityClass->id, 
                    'attribute_class_id' => $modelAttributeClass->id, 
                    'entity_id' => $trait_value_ids, 
                    'status' => 'active'
                ])
                ->asArray()
                ->all();
            
            foreach($trait_names as $trait_name) {
                $item_traits[$trait_k]['value'][$trait_name['entity_id']]['name'] = $trait_name['value'];

                if(\common\modules\inventory\models\InventoryItemTraitValue::find()->where(['item_id' => $id, 'trait_value_id' => $trait_name['entity_id'], 'status' => 'active'])->exists()) {
                    $item_traits[$trait_k]['value'][$trait_name['entity_id']]['is_selected'] = 1;
                } else {
                    $item_traits[$trait_k]['value'][$trait_name['entity_id']]['is_selected'] = 0;
                }
            }
        }
        
        if (Yii::$app->request->isPost) {
            if (($model->sku != Yii::$app->request->post('InventoryItem')['sku']) || 
                ($model->quantity != Yii::$app->request->post('InventoryItem')['quantity']) || 
                ($model->selling_price != Yii::$app->request->post('InventoryItem')['selling_price']) || 
                ($model->remarks != Yii::$app->request->post('InventoryItem')['remarks'])
            ) {
                $log_flag = true;
            }

            if ($item_id = $model->saveItem(Yii::$app->request->post(), $id)) {
                if ($log_flag === true) {
                    $eventInventoryEvents = new \common\modules\inventory\components\InventoryEvents;
                    $eventInventoryEvents->eventModInventoryLogChange($item_id, 'update');
                }

                return $this->redirect(['view', 'id' => $item_id]);   
            }
        }

        $data['model'] = $model;
        $data['item_traits'] = $item_traits;

        return $this->render('update', [
            'data' => $data,
        ]);
    }

    /**
     * Deletes an existing InventoryItem model.
     * If deletion is successful, the browser will be redirected to the 'index' page.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionDelete($id)
    {
        $this->findModel($id)->softDelete();

        $eventInventoryEvents = new \common\modules\inventory\components\InventoryEvents;
        $eventInventoryEvents->eventModInventoryLogChange($id, 'delete');

        return $this->redirect(['index']);
    }

    /**
     * Finds the InventoryItem model based on its primary key value.
     * If the model is not found, a 404 HTTP exception will be thrown.
     * @param integer $id
     * @return InventoryItem the loaded model
     * @throws NotFoundHttpException if the model cannot be found
     */
    protected function findModel($id)
    {
        if (($model = InventoryItem::findOne($id)) !== null) {
            return $model;
        }

        throw new NotFoundHttpException('The requested page does not exist.');
    }

    public function actionDuplicate($id)
    {
        $data = array();
        $model = $this->findModel($id);
        $item_traits = array();

        $traits = \common\modules\shop\models\ShopTrait::getAll('id', 'name', true, 'trait', 'name');

        $modelEntityClass = CoreEntityClass::findOne(['name' => 'trait_value', 'status' => 'active']);
        $modelAttributeClass = CoreAttributeClass::findOne(['name' => 'name', 'status' => 'active']);

        foreach ($traits as $trait_k => $trait_v) {
            $trait_value_ids = array();

            $item_traits[$trait_k]['name'] = $trait_v;

            if(\common\modules\inventory\models\InventoryItemTraitValue::find()->where(['item_id' => $id, 'trait_id' => $trait_k, 'status' => 'active'])->exists()) {
                $item_traits[$trait_k]['is_selected'] = 1;
            } else {
                $item_traits[$trait_k]['is_selected'] = 0;
            }

            $trait_values = \common\modules\shop\models\ShopTraitValue::find()->where(['trait_id' => $trait_k, 'status' => 'active'])->asArray()->all();
        
            foreach($trait_values as $trait_value) {
                array_push($trait_value_ids, $trait_value['id']);
            }
        
            $trait_value_ids = array_values($trait_value_ids);

            $trait_names = CoreValue::find()
                ->select('entity_id, value')
                ->where([
                    'entity_class_id' => $modelEntityClass->id, 
                    'attribute_class_id' => $modelAttributeClass->id, 
                    'entity_id' => $trait_value_ids, 
                    'status' => 'active'
                ])
                ->asArray()
                ->all();
            
            foreach($trait_names as $trait_name) {
                $item_traits[$trait_k]['value'][$trait_name['entity_id']]['name'] = $trait_name['value'];

                if(\common\modules\inventory\models\InventoryItemTraitValue::find()->where(['item_id' => $id, 'trait_value_id' => $trait_name['entity_id'], 'status' => 'active'])->exists()) {
                    $item_traits[$trait_k]['value'][$trait_name['entity_id']]['is_selected'] = 1;
                } else {
                    $item_traits[$trait_k]['value'][$trait_name['entity_id']]['is_selected'] = 0;
                }
            }
        }

        $data['model'] = $model;
        $data['item_traits'] = $item_traits;

        if (Yii::$app->request->isPost) {
            if ($item_id = $model->saveItem(Yii::$app->request->post())) {
                $eventInventoryEvents = new \common\modules\inventory\components\InventoryEvents;
                $eventInventoryEvents->eventModInventoryLogChange($item_id, 'create');

                return $this->redirect(['view', 'id' => $item_id]);
            } else {
                return $this->render('duplicate', [
                    'data' => $data,
                ]);    
            }
        } else {
            unset($data['model']['id']);

            $data['model']['sku'] = $data['model']['sku'] . '-COPY';

            return $this->render('duplicate', [
                'data' => $data,
            ]);
        }
    }
}
