<?php

namespace common\modules\mailer\controllers\backend;

use Yii;
use yii\data\ActiveDataProvider;
use yii\data\ArrayDataProvider;
use yii\web\Controller;
use yii\web\NotFoundHttpException;
use yii\helpers\ArrayHelper;
use yii\web\UploadedFile;

use common\modules\mailer\models\MailerContact;
use common\modules\mailer\models\MailerContactSearch;
use common\modules\core\models\FormFileUpload;  
use common\modules\core\models\CoreLanguage;
use common\modules\core\models\CoreEntityClass;
use common\modules\core\models\CoreAttributeClass;
use common\modules\core\models\CoreEntityAttribute;
use common\modules\core\models\CoreValue;

/**
 * ContactController implements the CRUD actions for MailerContact model.
 */
class ContactController extends Controller
{
    /**
     * {@inheritdoc}
     */
    public function behaviors()
    {
        return [
            'verbs' => [
                'class' => \yii\filters\VerbFilter::className(),
                'actions' => [
                    'delete' => ['POST'],
                ],
            ],
            'access' => [
                'class' => \yii\filters\AccessControl::className(),
                'only' => ['index', 'view', 'create', 'update', 'delete', 'chart', 'import', 'leads-calendar'],
                'rules' => [
                    [
                        'allow' => true,
                        'actions' => ['index', 'view', 'create', 'update', 'delete', 'chart', 'import', 'leads-calendar'],
                        'roles' => ['@'],
                        'matchCallback' => function ($rule, $action) {
                            return \common\modules\user\components\GenesysUser::isAllowed(Yii::$app->user->identity->role_id, Yii::$app->controller->module->id . '/' . Yii::$app->controller->id);
                        },
                    ],
                ],
            ],
        ];
    }

    /**
     * Lists all MailerContact models.
     * @return mixed
     */
    public function actionIndex()
    {
        $searchModel = new MailerContactSearch();
        $dataProvider = $searchModel->search(Yii::$app->request->queryParams);

        return $this->render('index', [
            'searchModel' => $searchModel,
            'dataProvider' => $dataProvider,
        ]);
    }

    /**
     * Displays a single MailerContact model.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionView($id)
    {
        return $this->render('view', [
            'model' => $this->findModel($id),
        ]);
    }

    /**
     * Creates a new MailerContact model.
     * If creation is successful, the browser will be redirected to the 'view' page.
     * @return mixed
     */
    public function actionCreate()
    {
        $model = new MailerContact();
        $data = array();

        if (Yii::$app->request->isPost) {
            if ($contact_id = $model->saveContact(Yii::$app->request->post())) {
                return $this->redirect(['view', 'id' => $contact_id]);
            }
        }

        $data['model'] = $model;

        return $this->render('create', [
            'data' => $data,
        ]);
    }

    /**
     * Updates an existing MailerContact model.
     * If update is successful, the browser will be redirected to the 'view' page.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionUpdate($id, $notif = null)
    {
        $model = $this->findModel($id);
        $data = array();

        if (isset($notif) && !empty($notif)) {
            \common\modules\site\models\SiteNotification::markRead($notif);
        }

        if (Yii::$app->request->isPost) {
            if ($contact_id = $model->saveContact(Yii::$app->request->post(), $id)) {
                return $this->redirect(['view', 'id' => $contact_id]);
            }
        }

        $data['model'] = $model;

        return $this->render('update', [
            'data' => $data,
        ]);
    }

    /**
     * Deletes an existing MailerContact model.
     * If deletion is successful, the browser will be redirected to the 'index' page.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionDelete($id)
    {
        $this->findModel($id)->softDelete();

        return $this->redirect(['index']);
    }

    /**
     * Finds the MailerContact model based on its primary key value.
     * If the model is not found, a 404 HTTP exception will be thrown.
     * @param integer $id
     * @return MailerContact the loaded model
     * @throws NotFoundHttpException if the model cannot be found
     */
    protected function findModel($id)
    {
        if (($model = MailerContact::findOne($id)) !== null) {
            return $model;
        }

        throw new NotFoundHttpException('The requested page does not exist.');
    }

    public function actionChart()
    {
        return $this->render('chart');
    }

    public function actionImport($type = 'mailer_contact', $tag = null)
    {
        $session = Yii::$app->session;

        $modelFormFileUpload = new FormFileUpload();
        $modelMailerContact = new MailerContact();
        $loop_counter = 0;
        $loop_break = false;

        if (Yii::$app->request->isPost) {
            $modelFormFileUpload->file = UploadedFile::getInstance($modelFormFileUpload, 'file');

            if (!empty($modelFormFileUpload->file)) {
                $filename = $modelFormFileUpload->file->baseName . '.' . $modelFormFileUpload->file->extension;

                if ($tempRelativePath = $modelFormFileUpload->upload('contact', $modelFormFileUpload->file->baseName . '.' . $modelFormFileUpload->file->extension)) {
                    $transaction = Yii::$app->db->beginTransaction();

                    try {
                        if (!$session->has('uploadedFileName')) {
                            $transaction->rollBack();

                            throw new HttpException(500, 'Unable to save file.');
                        }

                        if (!file_exists(Yii::getAlias('@assets') . '/data/' . $filename)) {
                            throw new NotFoundHttpException('File does not exist: ' . Yii::getAlias('@assets') . '/data/' . $filename);
                        }

                        $data = \moonland\phpexcel\Excel::import(Yii::getAlias('@assets') . '/data/' . $filename, [
                            'setFirstRecordAsKeys' => true, 
                        ]);

                        foreach ($data as $key => $value) {
                            $user_email = '';

                            if ($loop_break === true) {
                                break;
                            }

                            switch ($type) {
                                case 'mailer_contact':
                                    $user_email = strtolower(trim($value['email']));

                                    if (!isset($value['email']) || empty($value['email']) || MailerContact::find()->where(['email' => $user_email])->exists()) {
                                        continue 2;
                                    }

                                    $contact_id = $modelMailerContact->importMailerContact($value, $tag);

                                    if (!$contact_id) {
                                        $loop_break = true;
                                    } else {
                                        $loop_counter++;
                                    }

                                    break;

                                case 'user':
                                    $user_email = strtolower(trim($value['email']));

                                    if (!isset($value['email']) || empty($value['email']) || \common\modules\user\models\User::find()->where(['email' => $user_email])->exists()) {
                                        continue 2;
                                    }

                                    $user_id = $modelMailerContact->importUser($value);

                                    if (!$user_id) {
                                        $loop_break = true;
                                    } else {
                                        $loop_counter++;
                                    }

                                    break;
                                
                                default:
                                    
                                    break;
                            }
                        }

                        if ($loop_break === true) {
                            $transaction->rollBack();

                            Yii::$app->session->setFlash('error', "Unable to save record: " . json_encode($value));

                            return $this->render('import', [
                                'modelFormFileUpload' => $modelFormFileUpload,
                                'modelMailerContact' => $modelMailerContact,
                            ]);
                        } else {
                            $transaction->commit();

                            Yii::$app->session->setFlash('success', $loop_counter . " records saved successfully.");

                            return $this->render('import', [
                                'modelFormFileUpload' => $modelFormFileUpload,
                                'modelMailerContact' => $modelMailerContact,
                            ]);
                        }
                    } catch (\Exception $e) {
                        $transaction->rollBack();

                        throw $e;
                    }
                }
            }

        }

        return $this->render('import', [
            'modelFormFileUpload' => $modelFormFileUpload,
            'modelMailerContact' => $modelMailerContact,
        ]);
    }
    
    public function actionLeadsCalendar()
    {
        $modelMailerContact = new MailerContact();

        $modelMailerContact = MailerContact::find()->where(['status' => 'active'])->asArray()->all();
        $mailer_contact_metadata = ArrayHelper::getColumn($modelMailerContact, 'metadata');

        $leads_calendar = array();

        foreach ($mailer_contact_metadata as $key => $value) {
            $metadata_arr = json_decode($value, true);

            if (isset($metadata_arr['follow_up_date']) && !empty($metadata_arr['follow_up_date'])) {
                array_push(
                    $leads_calendar, 
                    [
                        'id' => $modelMailerContact[$key]['id'], 
                        'title' => $modelMailerContact[$key]['first_name'] . ' ' . $modelMailerContact[$key]['last_name'], 
                        'start' => $metadata_arr['follow_up_date'],
                        'className' => isset(Yii::$app->params['contact_metadata']['lead_status']['css']['class'][isset($metadata_arr['lead_status']) ? $metadata_arr['lead_status'] : '']) ? Yii::$app->params['contact_metadata']['lead_status']['css']['class'][isset($metadata_arr['lead_status']) ? $metadata_arr['lead_status'] : ''] : '',
                    ]
                );
            }
        }

        return $this->render('leads-calendar', [
            'leads_calendar' => $leads_calendar,
        ]);
    }
}
