<?php

namespace common\modules\shop\controllers\backend;

use Yii;
use yii\data\ActiveDataProvider;
use yii\data\ArrayDataProvider;
use yii\web\Controller;
use yii\web\NotFoundHttpException;
use Dompdf\Dompdf;
use Dompdf\Options;

use common\modules\shop\models\ShopOrder;
use common\modules\shop\models\ShopOrderSearch;
use common\modules\shop\models\ShopOrderItem;
use common\modules\shop\models\ShopOrderStatus;
use common\modules\shop\models\ShopTrait;
use common\modules\shop\models\ShopProductTraitValue;
use common\modules\shop\models\ShopVoucher;
use common\modules\shop\models\ShopProduct;
use common\modules\core\models\CoreSite;
use common\modules\core\models\CoreLanguage;
use common\modules\core\models\CoreEntityClass;
use common\modules\core\models\CoreAttributeClass;
use common\modules\core\models\CoreEntityAttribute;
use common\modules\core\models\CoreValue;
use common\modules\user\models\User;
use common\modules\site\components\Email;

/**
 * OrderController implements the CRUD actions for ShopOrder model.
 */
class OrderController extends Controller
{
    /**
     * {@inheritdoc}
     */
    public function behaviors()
    {
        return [
            'verbs' => [
                'class' => \yii\filters\VerbFilter::className(),
                'actions' => [
                    'delete' => ['POST'],
                ],
            ],
            'access' => [
                'class' => \yii\filters\AccessControl::className(),
                'only' => ['index', 'view', 'create', 'update', 'delete', 'ajax', 'chart'],
                'rules' => [
                    [
                        'allow' => true,
                        'actions' => ['index', 'view', 'create', 'update', 'delete', 'ajax', 'chart'],
                        'roles' => ['@'],
                        'matchCallback' => function ($rule, $action) {
                            return \common\modules\user\components\GenesysUser::isAllowed(Yii::$app->user->identity->role_id, Yii::$app->controller->module->id . '/' . Yii::$app->controller->id);
                        },
                    ],
                ],
            ],
        ];
    }

    /**
     * Lists all ShopOrder models.
     * @return mixed
     */
    public function actionIndex()
    {
        $searchModel = new ShopOrderSearch();
        $dataProvider = $searchModel->search(Yii::$app->request->queryParams);

        return $this->render('index', [
            'searchModel' => $searchModel,
            'dataProvider' => $dataProvider,
        ]);
    }

    /**
     * Displays a single ShopOrder model.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionView($id)
    {
        return $this->render('view', [
            'model' => $this->findModel($id),
            'modelOrderItem' => ShopOrderItem::findAll(['order_id' => $id, 'status' => 'active']),
        ]);
    }

    /**
     * Creates a new ShopOrder model.
     * If creation is successful, the browser will be redirected to the 'view' page.
     * @return mixed
     */
    public function actionCreate()
    {
        $session = Yii::$app->session;

        $model = new ShopOrder();
		$modelOrderItem = [new ShopOrderItem];
		$modelTraits = new ShopTrait;
        $modelCoreValue = new CoreValue;
        $modelVoucher = new ShopVoucher();
		$vouchers = array();
        $traits = array();
        $data = array();

        $modelVoucher = ShopVoucher::find()->where(['status' => 'active'])->asArray()->all();

        foreach ($modelVoucher as $voucherKey => $voucherValue) {
            $vouchers[$voucherValue['id']] = $voucherValue['discount'] . (($voucherValue['discount_percent'] == 'yes') ? '%' : '') . ' - ' . $voucherValue['code'];
        }

        $modelTraits = ShopTrait::find()->where(['status' => 'active'])->asArray()->all();

        if(!empty($modelTraits)) {
            $result = \yii\helpers\ArrayHelper::getColumn($modelTraits, 'id');
        }

        if (Yii::$app->request->isPost) {
            if ($order_id = $model->saveOrder(Yii::$app->request->post())) {
                return $this->redirect(['view', 'id' => $order_id]);   
            }
        }

        $data['model'] = $model;
        $data['modelOrderItem'] = (empty($modelOrderItem)) ? [new ShopOrderItem] : $modelOrderItem;
        $data['vouchers'] = $vouchers;

        return $this->render('create', [
            'data' => $data,
        ]);
    }

    /**
     * Updates an existing ShopOrder model.
     * If update is successful, the browser will be redirected to the 'view' page.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionUpdate($id)
    {
        $session = Yii::$app->session;
        
        $model = $this->findModel($id);
        $modelVoucher = new ShopVoucher();
        $modelOrderItem = [new ShopOrderItem];
        $vouchers = array();
        $data = array();

        $modelOrderItem = ShopOrderItem::findAll(['order_id' => $id, 'status' => 'active']);

        $modelVoucher = ShopVoucher::find()->where(['status' => 'active'])->asArray()->all();

        foreach ($modelVoucher as $voucherKey => $voucherValue) {
            $vouchers[$voucherValue['id']] = $voucherValue['discount'] . (($voucherValue['discount_percent'] == 'yes') ? '%' : '') . ' - ' . $voucherValue['code'];
        }
        
        if (Yii::$app->request->isPost) {
            if ($order_id = $model->saveOrder(Yii::$app->request->post(), $id)) {
                return $this->redirect(['view', 'id' => $order_id]);   
            }
        }

        $data['model'] = $model;
        $data['modelOrderItem'] = (empty($modelOrderItem)) ? [new ShopOrderItem] : $modelOrderItem;
        $data['vouchers'] = $vouchers;

        return $this->render('update', [
            'data' => $data,
        ]);
    }

    /**
     * Deletes an existing ShopOrder model.
     * If deletion is successful, the browser will be redirected to the 'index' page.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionDelete($id)
    {
        $this->findModel($id)->softDelete();

        return $this->redirect(['index']);
    }

    /**
     * Finds the ShopOrder model based on its primary key value.
     * If the model is not found, a 404 HTTP exception will be thrown.
     * @param integer $id
     * @return ShopOrder the loaded model
     * @throws NotFoundHttpException if the model cannot be found
     */
    protected function findModel($id)
    {
        if (($model = ShopOrder::findOne($id)) !== null) {
            return $model;
        }

        throw new NotFoundHttpException('The requested page does not exist.');
    }

    public function actionAjax()
    {
        $product_traits = array();
        $modelShopProductTrait = new ShopProductTraitValue();
        $productid = Yii::$app->request->post('product_id');
        
        if (isset($productid)) {
            $modelShopProductTrait = ShopProductTraitValue::find()->select('trait_id,trait_value_id')->where(['product_id' => $productid, 'status' => 'active'])->distinct()->asArray()->all();
            if(!empty($modelShopProductTrait)) {
                $traitValueArr = \yii\helpers\ArrayHelper::getColumn($modelShopProductTrait,'trait_value_id');

                $modelEntityClass = CoreEntityClass::findOne(['name' => 'trait_value', 'status' => 'active']);
                $modelAttributeClass = CoreAttributeClass::findOne(['name' => 'name', 'status' => 'active']);
                $traitNamesArray = CoreValue::find()->select('entity_id, value')->where(['entity_class_id' => $modelEntityClass->id, 'attribute_class_id' => $modelAttributeClass->id, 'entity_id' => $traitValueArr ,'status' => 'active'])->asArray()->all();

                foreach($modelShopProductTrait as $key => $value) {                
                    foreach($traitNamesArray as $k => $v){
                        if($value['trait_value_id'] == $v['entity_id']) {
                            $product_traits[$value['trait_id']][$v['entity_id']] = $v['value'];
                        }
                    }
                }
                $traitValueNamesArr = \yii\helpers\ArrayHelper::map($modelShopProductTrait,'entity_id','value');
            } else {
                
            }           
            $test = $product_traits;
        } else {
            $test = "Ajax failedas";
        }
        
        return \yii\helpers\Json::encode($test);
    }

    public function actionSendOrderConfirmation($order_id)
    {
        if (!isset($order_id) || empty($order_id)) {
            throw new \yii\web\BadRequestHttpException("Unable to send order confirmation.", 400);
        }

        $modelShopOrderArr = new ShopOrder();
        $modelShopOrderItemArr = new ShopOrderItem();
        $modelUserArr = new User();
        $sent = false;

        if (!ShopOrder::find()->where(['id' => $order_id, 'status' => 'active'])->exists()) {
            throw new NotFoundHttpException("Order not found.");
        }

        $modelShopOrderArr = ShopOrder::find()->where(['id' => $order_id, 'status' => 'active'])->asArray()->one();
        $modelShopOrderItemArr = ShopOrderItem::find()->where(['order_id' => $modelShopOrderArr['id'], 'status' => 'active'])->asArray()->all();
        $modelUserArr = User::find()->where(['id' => $modelShopOrderArr['user_id'], 'status' => 'active'])->asArray()->one();

        $modelUserArr['password_hash'] = $modelUserArr['auth_key'];

        if ($modelShopOrderArr['order_status_id'] == ShopOrderStatus::getAttr(['id'], ['name' => 'New'])) {
            $sent = Email::send(
                'order-confirmation-customer', 
                [
                    'html' => 'orderConfirmationCustomer-html', 
                    'text' => 'orderConfirmationCustomer-text'
                ], 
                [
                    'to' => $modelUserArr['email'], 
                    'from' => Yii::$app->params['email']['from'][Yii::$app->params['mode']]
                ], 
                null, 
                ['user_data' => [
                    'user' => $modelUserArr, 
                    'order' => $modelShopOrderArr, 
                    'order_item' => $modelShopOrderItemArr, 
                    'payment_gateway' => Yii::$app->params['payment_gateway'],
                    'is_fallback' => false,
                    'is_manual' => true,
                ]], 
                CoreSite::getAttr(['code'], ['id' => $modelShopOrderArr['site_id']]), 
                CoreSite::getAttr(['code'], ['id' => $modelShopOrderArr['site_id']])
            );
        }

        if ($sent) {
            Yii::$app->session->setFlash('success', "Order confirmation sent successfully.");
        } else {
            Yii::$app->session->setFlash('error', "Unable to send order confirmation.");
        }

        return $this->redirect(['view', 
            'id' => $order_id
        ]);
    }

    public function actionChart()
    {
        return $this->render('chart');
    }
}
